import datetime, asyncio, pkg_resources
from boldigger.boldblast_coi import fasta_to_string, post_request, as_session, save_as_df, save_results, fasta_rewrite, excel_converter
from boldigger_cline.login import login
from tqdm import tqdm
from requests.exceptions import ReadTimeout
from requests.exceptions import ConnectionError

def main(username, password, fasta_path, output_path, query_length):

    ## import data needed for the requests
    certs = pkg_resources.resource_filename(__name__, 'data/certs.pem')

    ## define some variables needed for the layout
    session = login(username, password, certs)
    querys, sequences_names = fasta_to_string(fasta_path, query_length)

    ## request as long as there are querys left
    for query in tqdm(querys, desc = 'Requesting BOLD'):
        while True:
            try:
                tqdm.write('%s: Requesting BOLD. This will take a while.' % datetime.datetime.now().strftime("%H:%M:%S"))

                ## collect IDS result urls from BOLD
                links = post_request(query, session)

                ## download data from the fetched links
                tqdm.write('%s: Downloading results.' % datetime.datetime.now().strftime("%H:%M:%S"))
                tables = asyncio.run(as_session(links))

                ## parse the returned html
                tqdm.write('%s: Parsing html.' % datetime.datetime.now().strftime("%H:%M:%S"))
                result = save_as_df(tables, sequences_names[querys.index(query)])

                ## save results in resultfile
                tqdm.write('%s: Saving results.' % datetime.datetime.now().strftime("%H:%M:%S"))
                save_results(result, fasta_path, output_path)

            except (ValueError, ReadTimeout, ConnectionError):
                tqdm.write('%s: BOLD did not respond! Retrying.' % datetime.datetime.now().strftime("%H:%M:%S"))
                continue
            break

        ## remove found OTUS from fasta and write it into a new one
        tqdm.write('%s: Removing finished OTUs from fasta.' % datetime.datetime.now().strftime("%H:%M:%S"))
        fasta_rewrite(fasta_path, query_length)

    ## convert results to excel when download is finished
    tqdm.write('%s: Converting the data to excel.' % datetime.datetime.now().strftime("%H:%M:%S"))
    excel_converter(fasta_path, output_path)

    print('%s: Done. Close to continue.' % datetime.datetime.now().strftime("%H:%M:%S"))



import pkg_resources, argparse
from boldigger_cline import (
    boldblast_coi,
    boldblast_its,
    boldblast_rbcl,
    additional_data,
    first_hit,
    jamp_hit,
    digger_sort,
    api_verification,
)


## main function to run and control the flow of bldigger-cline
def main():
    ## collect and parse the cline arguments
    parser = argparse.ArgumentParser(
        prog="boldigger-cline",
        description="BOLDigger as a command line tool, https://github.com/DominikBuchner/BOLDigger-commandline",
    )
    subparsers = parser.add_subparsers(dest="function")

    ## group parsers that accept the same arguments
    ## identification engines
    parser_ie_coi = subparsers.add_parser("ie_coi", help="COI identification engine")
    parser_ie_its = subparsers.add_parser("ie_its", help="ITS identification engine")
    parser_ie_rbcl = subparsers.add_parser("ie_rbcl", help="rbcl identification engine")

    for subparser in [parser_ie_coi, parser_ie_its, parser_ie_rbcl]:
        subparser.add_argument("username", help="Username for boldsystems login")
        subparser.add_argument("password", help="Password for boldsystems login")
        subparser.add_argument(
            "fasta_path", help="Path to the fasta file to be blasted"
        )
        subparser.add_argument("output_folder", help="Path to the output folder")
        subparser.add_argument(
            "batch_size",
            type=int,
            default=50,
            nargs="?",
            help="Batch size to be blasted.",
        )

    ## metadata code
    parser_add_data = subparsers.add_parser(
        "add_metadata", help="additional data download"
    )

    ## top hit selection
    parser_first_hit = subparsers.add_parser(
        "first_hit", help="use the first hit as top hit"
    )
    parser_jamp_hit = subparsers.add_parser(
        "jamp_hit", help="determine the top hit with the JAMP method"
    )
    parser_digger_hit = subparsers.add_parser(
        "digger_hit", help="determine the top hit with the BOLDigger method"
    )

    for subparser in [
        parser_add_data,
        parser_first_hit,
        parser_jamp_hit,
        parser_digger_hit,
    ]:
        subparser.add_argument("xlsx_path", help="Path to the BOLDresults file")

    ## add a parser for the api verfication
    parser_api_verification = subparsers.add_parser(
        "api_verification", help="run the api verification"
    )
    parser_api_verification.add_argument(
        "xlsx_path", help="Path to the BOLDResults file"
    )
    parser_api_verification.add_argument(
        "fasta_path", help="Path to the fasta file belonging to the BOLDResults file"
    )

    ## add version control
    parser.add_argument("--version", action="version", version="2.2.1")

    ## parse the arguments
    args = parser.parse_args()

    ## search engine for coi
    if args.function == "ie_coi":
        boldblast_coi.main(
            args.username,
            args.password,
            args.fasta_path,
            args.output_folder,
            args.batch_size,
        )

    ## search engine for its
    if args.function == "ie_its":
        boldblast_its.main(
            args.username,
            args.password,
            args.fasta_path,
            args.output_folder,
            args.batch_size,
        )

    ## search engine for rbcL
    if args.function == "ie_rbcl":
        boldblast_rbcl.main(
            args.username,
            args.password,
            args.fasta_path,
            args.output_folder,
            args.batch_size,
        )

    ## additional data
    if args.function == "add_metadata":
        additional_data.main(args.xlsx_path)

    ## hit selection
    if args.function == "first_hit":
        first_hit.main(args.xlsx_path)

    if args.function == "jamp_hit":
        jamp_hit.main(args.xlsx_path)

    if args.function == "digger_hit":
        digger_sort.main(args.xlsx_path)

    if args.function == "api_verification":
        api_verification.main(args.xlsx_path, args.fasta_path)


## run only if called as a toplevel script
if __name__ == "__main__":
    main()



import openpyxl, datetime
import pandas as pd
from boldigger.first_hit import first_hit

## main function to control the flow
def main(xlsx_path):

    print('%s: Opening resultfile.' % datetime.datetime.now().strftime("%H:%M:%S"))

    ## run first hit function
    print('%s: Filtering data.' % datetime.datetime.now().strftime("%H:%M:%S"))
    first_hit(xlsx_path)

    print('%s: Saving result to new tab.' % datetime.datetime.now().strftime("%H:%M:%S"))
    print('%s: Done.' % datetime.datetime.now().strftime("%H:%M:%S"))



import openpyxl, requests_html, datetime, sys
from more_itertools import unique_everseen
from boldigger.boldblast_coi import slices
from bs4 import BeautifulSoup as BSoup
from boldigger.additional_data import retrieve_process_ids, save_results
from tqdm import tqdm


## function to loop through the process IDs and call the BOLD API  for additional data
## will return a dict in form of {Process ID: [BOLD Record ID, BIN, Sex, Life stage, Country, Identifier, Identification method, Institution storing]}
def get_data(process_ids):
    ## slice the process ids in parts of 100 to get the maximum out of API calls
    process_ids = list(slices(process_ids, 100))

    ## open a new html session
    with requests_html.HTMLSession() as session:
        ## initialize the final dict output
        process_id_dict = {}

        for id_pack in tqdm(process_ids, desc="Download status"):
            ## request the data, retry if the request failes
            while True:
                try:
                    resp = session.get(
                        "http://www.boldsystems.org/index.php/API_Public/specimen?ids="
                        + "|".join(id_pack)
                    )
                except:
                    continue
                break

            ## use beautifulsoup to find all records in the response
            soup = BSoup(resp.text, "xml")
            records = soup.find_all("record")

            ## loop through records to extract interesting data
            for count in range(len(records)):
                tags = [
                    "processid",
                    "record_id",
                    "bin_uri",
                    "sex",
                    "lifestage",
                    "country",
                    "identification_provided_by",
                    "identification_method",
                    "institution_storing",
                ]

                ## search the data encapsuled by each tag
                specimen_data = [
                    records[count].find(tag) if records[count].find(tag) != None else ""
                    for tag in tags
                ]

                ## extract the texts from these tags
                specimen_data = [
                    datapoint.text if datapoint != "" else ""
                    for datapoint in specimen_data
                ]

                ## append a link to the specimen page to the results if user is interested in looking up even more data
                specimen_data.append(
                    "http://www.boldsystems.org/index.php/MAS_DataRetrieval_OpenSpecimen?selectedrecordid="
                    + specimen_data[1]
                )

                ## put the data in a dictionary in form of processID: [listofinformation]
                process_dict = {specimen_data[0]: specimen_data[1:]}

                ## update the final output
                process_id_dict.update(process_dict)

    return process_id_dict


## main function to controll the flow
def main(xlsx_path):
    ## Give user output
    tqdm.write(
        "%s: Extracting process ID's." % datetime.datetime.now().strftime("%H:%M:%S")
    )

    ## catch any wrong file formats here to avoid crash
    try:
        wb, process_ids, type = retrieve_process_ids(xlsx_path)
    except ValueError:
        tqdm.write("Wrong file format. Close to continue.")
        sys.exit()

    ## collect the data from BOLD
    tqdm.write(
        "%s: Downloading additional data."
        % datetime.datetime.now().strftime("%H:%M:%S")
    )
    additional_data = get_data(process_ids)

    ## saving the data according to type
    tqdm.write("%s: Saving the results." % datetime.datetime.now().strftime("%H:%M:%S"))
    save_results(wb, additional_data, xlsx_path, type)

    ## get events (just closing) and values
    tqdm.write(
        "%s: Done. Close to continue." % datetime.datetime.now().strftime("%H:%M:%S")
    )



from boldigger.api_verification import tqdm_joblib, extract_data, request, refresh_data
import datetime, contextlib, requests_html, joblib, json, openpyxl, psutil
import PySimpleGUI as sg
import pandas as pd
import numpy as np
from tqdm import tqdm
from joblib import Parallel, delayed
from Bio.SeqIO.FastaIO import SimpleFastaParser
from boldigger.boldblast_coi import slices


## main function to control the flow of the program
def main(xlsx_path, fasta_path):
    ## collect the data
    print(
        "{}: Starting API verification.".format(
            datetime.datetime.now().strftime("%H:%M:%S")
        )
    )
    print(
        "{}: Collection OTUs without species level identification and high similarity.".format(
            datetime.datetime.now().strftime("%H:%M:%S")
        )
    )
    raw_data, data_to_check, seq_dict = extract_data(xlsx_path, fasta_path)

    if not raw_data.empty:
        print(
            "{}: Starting to query the API.".format(
                datetime.datetime.now().strftime("%H:%M:%S")
            )
        )

        session = requests_html.HTMLSession()

        with tqdm_joblib(
            tqdm(desc="Calling API", total=len(list(seq_dict.items())))
        ) as progress_bar:
            result = Parallel(n_jobs=psutil.cpu_count())(
                delayed(request)(item, session) for item in list(seq_dict.items())
            )

        ## remove all hits that did not match
        result = [res for res in result if res != None]
        print(
            "{}: Collected {} additional species names.".format(
                datetime.datetime.now().strftime("%H:%M:%S"), len(result)
            )
        )
        print(
            "{}: Starting to update the dataset.".format(
                datetime.datetime.now().strftime("%H:%M:%S")
            )
        )

        corrected_data = refresh_data(result, raw_data, data_to_check, session)

        print(
            "{}: Saving the corrected dataset.".format(
                datetime.datetime.now().strftime("%H:%M:%S")
            )
        )

        with pd.ExcelWriter(
            xlsx_path, mode="a", if_sheet_exists="replace", engine="openpyxl"
        ) as writer:
            corrected_data.to_excel(
                writer, sheet_name="BOLDigger hit - API corrected", index=False
            )

        # ## save output
        # wb = openpyxl.load_workbook(xlsx_path)
        # writer = pd.ExcelWriter(xlsx_path, engine = 'openpyxl')
        # writer.book = wb

        # corrected_data.to_excel(writer, sheet_name = 'BOLDigger hit - API corrected', index = False)
        # wb.save(xlsx_path)
        # writer.close()

        print("{}: Done.".format(datetime.datetime.now().strftime("%H:%M:%S")))
    else:
        print(
            "%s: No BOLDigger hits found. Please add these tophits first. Close to continue."
            % datetime.datetime.now().strftime("%H:%M:%S")
        )



import requests_html, json, os, sys
import PySimpleGUI as sg
from bs4 import BeautifulSoup as BSoup
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry


## function to login to bold
def login(username, password, certificate):
    ## start a new html session
    session = requests_html.HTMLSession()
    session.headers.update(
        {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.82 Safari/537.36"
        }
    )
    retry_strategy = Retry(
        total=15,
        status_forcelist=[400, 401, 403, 404, 413, 429, 500, 502, 503, 504],
        backoff_factor=1,
    )
    adapter = HTTPAdapter(max_retries=retry_strategy)
    session.mount("http://", adapter)

    ## data to push into the post request
    data = {
        "name": username,
        "password": password,
        "destination": "MAS_Management_UserConsole",
        "loginType": "",
    }

    ## send a post request to log into boldsystems.org
    session.post("https://boldsystems.org/index.php/Login", data=data)

    ## test if the login was successfull
    url = session.get("https://boldsystems.org/")
    soup = BSoup(url.text, "html.parser")
    content = soup.find(class_="site-navigation nav navbar-nav")
    tags = content.find_all("a")
    if tags[5].text != "Log out":
        print("Unable to login.\nPlease check your userdata.")
        sys.exit()
    else:
        print("Login successful.")

    ## return the session, not neccessary for this check but
    ## useful if you want to do other things with the login
    return session



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="boldigger-cline",
    version="2.2.3",
    author="Dominik Buchner",
    author_email="dominik.buchner524@googlemail.com",
    description="BOLDigger as a command-line tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DominikBuchner/BOLDigger-commandline",
    packages=setuptools.find_packages(),
    license="MIT",
    install_requires=["tqdm >= 4.32.2", "boldigger >= 2.2.1"],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    entry_points={
        "console_scripts": [
            "boldigger-cline = boldigger_cline.__main__:main",
        ]
    },
)



import openpyxl, datetime
import pandas as pd
from boldigger.jamp_hit import get_threshold, get_data, jamp_hit, save_results

## main function
def main(xlsx_path):

    print('%s: Opening resultfile.' % datetime.datetime.now().strftime("%H:%M:%S"))
    otu_dfs = get_data(xlsx_path)

    if otu_dfs != 'Wrong file':
        print('%s: Filtering data.' % datetime.datetime.now().strftime("%H:%M:%S"))
        jamp_hits = [jamp_hit(otu) for otu in otu_dfs]
        output = pd.concat(jamp_hits).reset_index(drop = True)

        print('%s: Saving result to new tab.' % datetime.datetime.now().strftime("%H:%M:%S"))
        save_results(xlsx_path, output)

        print('%s: Done.' % datetime.datetime.now().strftime("%H:%M:%S"))

    else:
        print('%s: Wrong file format.' % datetime.datetime.now().strftime("%H:%M:%S"))



import openpyxl, datetime
import pandas as pd
from boldigger.jamp_hit import get_data, jamp_hit, get_threshold
from boldigger.digger_sort import get_full_data, add_flags, save_results

## main function to control the flow
def main(xlsx_path):

    ## determine hits the JAMP way
    print('%s: Opening resultfile.' % datetime.datetime.now().strftime("%H:%M:%S"))
    otu_dfs = get_data(xlsx_path)

    if otu_dfs != 'Wrong file':
        print('%s: Filtering data for JAMP hits.' % datetime.datetime.now().strftime("%H:%M:%S"))
        jamp_hits = [jamp_hit(otu) for otu in otu_dfs]

        ## get the full data for flagging
        print('%s: Extracting additional data.' % datetime.datetime.now().strftime("%H:%M:%S"))
        full_data = get_full_data(xlsx_path)

        if get_full_data(xlsx_path) != 'No metadata':
            ## flag every hit
            print('%s: Flagging the hits.' % datetime.datetime.now().strftime("%H:%M:%S"))
            for i in range(len(jamp_hits)):
                jamp_hits[i]['Flags'] = add_flags(jamp_hits[i], full_data[i])

            ## save results
            print('%s: Saving result to new tab.' % datetime.datetime.now().strftime("%H:%M:%S"))
            output = pd.concat(jamp_hits).reset_index(drop = True)
            save_results(xlsx_path, output)

            print('%s: Done. Close to continue.' % datetime.datetime.now().strftime("%H:%M:%S"))

        else:
            print('%s: No additional data found. Run again after the additional data download. Close to continue.' % datetime.datetime.now().strftime("%H:%M:%S"))

    else:
        print('%s: Wrong file format. Close to continue.' % datetime.datetime.now().strftime("%H:%M:%S"))
